#ifndef URI_H_
#define URI_H_

#include <ne_uri.h>
#include <string>

namespace webdav
{

class URI
{
public:
	URI();
	URI(ne_uri);
	URI(const char* uri);
	virtual ~URI();

	URI& operator=(const URI& other);
	URI& operator=(const char* other);
	operator std::string() const;

	std::string GetScheme() const;
	std::string GetHost() const;
	unsigned int GetPort() const;
	std::string GetPath() const;
	std::string GetAuthInfo() const;

	void SetScheme(const char* scheme);
	void SetHost(const char* host);
	void SetPort(unsigned int port);
	void SetPath(const char* path);
	void SetAuthInfo(const char* userinfo);

	static unsigned int GetDefaultPort(const char* scheme);

private:
	ne_uri uri_;
};

inline URI::operator std::string() const
{
	char* buf = ne_uri_unparse(&uri_);
	std::string uri(buf);

	free(buf);

	return uri;
}

} // namespace WebDAV

#endif
