#ifndef WEBDAV_H_
#define WEBDAV_H_

#include "api.h"
#include ".\resource.h"
#include <fstream>
#include <Windows.h>
#include <Wincrypt.h>

namespace webdav
{

class WebDAVImpl;
class Resource;

enum NotifyStatus
{
	NOTIFY_LOOKUP,
	NOTIFY_CONNECTING,
	NOTIFY_CONNECTED,
	NOTIFY_SENDING,
	NOTIFY_RECVING
};

union in_addr;

union NotifyInfo
{
    struct /* ne_status_lookup */ {
        const char *hostname;
    } lu;

    struct /* ne_status_connecting */ {
        const char *hostname;
        const in_addr *address;
    } ci;

    struct /* ne_status_connected */ {
        const char *hostname;
    } cd;

    struct /* ne_status_sending and ne_status_recving */ {
        /* If total == 0, total is unknown; otherwise, 0 <= point
         * <= total.  */
        off_t progress, total;
    } sr;
};

enum Status
{
	OK,
	GenericError,
	LookupFaild,
	AuthenticationFaild,
	ProxyAuthenticationFaild,
	ConnectionFaild,
	TimeOut,
	PreconditionFaild,
	Retry,
};

enum SslCertificateVerificationFailures
{
	NotYetValid = 0x01,
	Expired     = 0x02,
	IdMismatch  = 0x04,
	Untrusted   = 0x08,
	BadChain    = 0x10,
	Revoked     = 0x20
};

typedef unsigned int UINT;

struct SslCertificate
{
	SslCertificate()
		: subject(NULL),
		issuer(NULL),
		identity(NULL)
	{
	}

    // ne_ssl_dname subj_dn, issuer_dn;
    PCCERT_CONTEXT subject;
	SslCertificate* issuer; 
    char *identity;
	
	SslCertificate(const SslCertificate& obj)
		: subject(NULL),
		issuer(NULL),
		identity(NULL)
	{
	}
};

struct WEBDAV_API WebDAV
{
	friend WebDAVImpl;
	
	WebDAV();
	virtual ~WebDAV();

	static int Init();
	static void Exit();

	// F
	virtual void SetUserName(const char* username);
	virtual void SetPassword(const char* password);
	virtual void SetUserAgent(const char* token);
	virtual void SetProxy(const char* host, UINT port, const char* username = NULL, const char* password = NULL);

	virtual bool IsConnected() const;

	// Method
	virtual bool Connect(const char* uri);
	virtual bool Connect(const char* scheme, const char* host);
	virtual bool GetList(const char* path, Resource** buf, size_t* buf_size);

	virtual bool Get(const char* rpath, const char* lpath);
	virtual bool Put(const char* rpath, const char* lpath);
	virtual bool MakeCollection(const char* path);
	virtual bool Delete(const char* path);
	virtual bool Move(const char* spath, const char* dpath, bool is_overwrite = false);
	virtual bool Copy(const char* spath, const char* dpath, bool is_overwrite = false);
	virtual bool GetFileProperty(const char* path);

	// Collection
	virtual bool OpenCol(const char* path, int& handle);
	virtual bool ReadCol(int handle, Resource& res);
	virtual void CloseCol(int handle);

	virtual void NotifyStatus(NotifyStatus status, NotifyInfo* info);
	virtual void NotifyProgress(__int64 progress, __int64 total);
	virtual int NotifyResponse(const char* buffer, size_t buflen);
	virtual size_t NotifyRequest(char* buffer, size_t buflen);
	virtual int VerifyServerCertificate(int failures, SslCertificate* cert);

	PCCERT_CONTEXT GetSslServerCert();

	Status GetLastStatus() const;
	void DebugInit(FILE *stream, int mask);
	virtual void ExitThread();

private:
	WebDAVImpl* pimpl_;
};

} // namespace webdav

#endif